"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cheerio_1 = tslib_1.__importDefault(require("cheerio"));
const fs = tslib_1.__importStar(require("fs-extra"));
const os_1 = require("os");
const path = tslib_1.__importStar(require("path"));
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
const tough = require("tough-cookie");
const tough_cookie_filestore_1 = tslib_1.__importDefault(require("tough-cookie-filestore"));
const url = tslib_1.__importStar(require("url"));
const utils_1 = require("./utils");
let version = '';
try {
    version = require('../package.json').version;
}
catch (err) { }
const defaultCookiePath = path.join(os_1.homedir(), '.hackmd', 'cookies.json');
const defaultConfig = {
    cookiePath: defaultCookiePath,
    serverUrl: 'https://hackmd.io',
    enterprise: true
};
function encodeFormComponent(form) {
    return Object.entries(form).map(([key, value]) => `${encodeURIComponent(key)}=${encodeURIComponent(value)}`).join('&');
}
var ExportType;
(function (ExportType) {
    ExportType[ExportType["PDF"] = 0] = "PDF";
    ExportType[ExportType["SLIDE"] = 1] = "SLIDE";
    ExportType[ExportType["MD"] = 2] = "MD";
    ExportType[ExportType["HTML"] = 3] = "HTML";
})(ExportType = exports.ExportType || (exports.ExportType = {}));
/**
 * codimd API Client
 */
class API {
    constructor(config = {}) {
        const { serverUrl, cookiePath, enterprise } = utils_1.defaults(config, defaultConfig);
        fs.ensureFileSync(cookiePath);
        const jar = new tough_cookie_filestore_1.default(cookiePath);
        const fetch = require('fetch-cookie')(node_fetch_1.default, new tough.CookieJar(jar));
        this._fetch = fetch;
        this.serverUrl = url.parse(serverUrl).href;
        this.enterprise = enterprise;
    }
    async login(email, password) {
        await this.fetch(url.resolve(this.serverUrl, 'login'), {
            method: 'post',
            body: encodeFormComponent({ email, password }),
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
            })
        });
    }
    async loginLdap(username, password) {
        await this.fetch(url.resolve(this.serverUrl, 'auth/ldap'), {
            method: 'post',
            body: encodeFormComponent({ username, password }),
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
            })
        });
    }
    async logout() {
        const response = await this.fetch(url.resolve(this.serverUrl, 'logout'), {
            method: this.enterprise ? 'POST' : 'GET',
            headers: await this.wrapHeaders({
                'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
            })
        });
        return response.status === 200;
    }
    async isLogin() {
        try {
            const data = await this.getMe();
            return data.status === 'ok';
        }
        catch (err) {
            return false;
        }
    }
    async getMe() {
        const response = await this.fetch(url.resolve(this.serverUrl, 'me'), this.defaultFetchOptions);
        return response.json();
    }
    async getHistory() {
        const response = await this.fetch(url.resolve(this.serverUrl, 'history'), this.defaultFetchOptions);
        return response.json();
    }
    async newNote(body, options) {
        let response;
        if (this.enterprise) {
            let newNoteUrl;
            if (options === null || options === void 0 ? void 0 : options.team) {
                newNoteUrl = url.resolve(this.serverUrl, `team/${options.team}/new`);
            }
            else {
                newNoteUrl = url.resolve(this.serverUrl, 'new');
            }
            response = await this.fetch(newNoteUrl, {
                method: 'POST',
                body: encodeFormComponent({ content: body }),
                headers: await this.wrapHeaders({
                    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
                })
            });
        }
        else {
            const contentType = 'text/markdown;charset=UTF-8';
            response = await this.fetch(url.resolve(this.serverUrl, 'new'), {
                method: 'POST',
                body,
                headers: await this.wrapHeaders({
                    'Content-Type': contentType
                })
            });
        }
        if (response.status === 200) {
            return response.url;
        }
        else {
            throw new Error('Create note failed');
        }
    }
    async listNotes() {
        // https://hackmd.io/api/overview?v=1627369059523
        if (this.enterprise) {
            const response = await this.fetch(url.resolve(this.serverUrl, 'api/overview'), this.defaultFetchOptions);
            return response.json();
        }
        else {
            throw new Error('Not support');
        }
    }
    async listTeamNotes(teamPath) {
        if (this.enterprise) {
            return this.fetch(url.resolve(this.serverUrl, `api/overview/team/${teamPath}`), this.defaultFetchOptions).then(res => res.json());
        }
        else {
            throw new Error('Not support');
        }
    }
    async exportRes(noteId, type) {
        let res;
        switch (type) {
            case ExportType.PDF:
                res = await this.fetch(url.resolve(this.serverUrl, `${noteId}/pdf`), this.defaultFetchOptions);
                break;
            case ExportType.HTML:
                res = await this.fetch(url.resolve(this.serverUrl, `s/${noteId}`), this.defaultFetchOptions);
                break;
            case ExportType.SLIDE:
                res = await this.fetch(url.resolve(this.serverUrl, `${noteId}/slide`), this.defaultFetchOptions);
                break;
            case ExportType.MD:
            default:
                res = await this.fetch(url.resolve(this.serverUrl, `${noteId}/download`), this.defaultFetchOptions);
        }
        return res;
    }
    async exportString(noteId, type) {
        const res = await this.exportRes(noteId, type);
        return res.text();
    }
    async exportStream(noteId, type) {
        const res = await this.exportRes(noteId, type);
        return res.body;
    }
    async getTeams() {
        let data;
        try {
            data = await this.getMe();
        }
        catch (err) {
            return [];
        }
        return data.teams || [];
    }
    get fetch() {
        return this._fetch;
    }
    get domain() {
        return url.parse(this.serverUrl).host;
    }
    get defaultFetchOptions() {
        return {
            headers: {
                'User-Agent': `HackMD API Client ${version} Node.js`
            }
        };
    }
    async wrapHeaders(headers) {
        if (this.enterprise) {
            const csrf = await this.loadCSRFToken();
            return Object.assign(Object.assign({}, headers), { 'User-Agent': `HackMD API Client ${version} Node.js`, 'X-XSRF-Token': csrf });
        }
        else {
            return Object.assign(Object.assign({}, headers), { 'User-Agent': `HackMD API Client ${version} Node.js` });
        }
    }
    async loadCSRFToken() {
        const html = await this.fetch(this.serverUrl).then(r => r.text());
        const $ = cheerio_1.default.load(html);
        return $('meta[name="csrf-token"]').attr('content') || '';
    }
}
exports.default = API;
